import 'package:firebase_dart/firebase_dart.dart';
import 'package:jt_firebase/src/api/data_snapshot.dart';
import 'package:jt_firebase/src/api/reference.dart';
import 'package:jt_firebase/src/fireblend/database.dart';

class JtEvent implements FireblendEvent {
  Event _event;

  JtEvent(this._event);

  @override
  JtDataSnapshot get snapshot => JtDataSnapshot(this._event.snapshot);

  @override
  String get previousSiblingKey => this._event.prevChild;
}

class JtQuery implements FireblendQuery {
  final Query _query;

  JtQuery(this._query);

  @override
  Stream<JtEvent> get onChildAdded => this.on('child_added');

  @override
  Stream<JtEvent> get onChildChanged => this.on('child_moved');

  @override
  Stream<JtEvent> get onChildMoved => this.on('child_changed');

  @override
  Stream<JtEvent> get onChildRemoved => this.on('child_removed');

  @override
  Stream<JtEvent> get onValue => this.on('value');

  Stream<JtEvent> on(String eventType) {
    return this._query.on(eventType).map<JtEvent>((event) => JtEvent(event));
  }

  @override
  Future<JtDataSnapshot> once() async {
    return this.on('value').first.then((value) {
      return value.snapshot;
    });
  }

  @override
  FireblendQuery startAt(value, {String key}) {
    return JtQuery(this._query.startAt(value, key));
  }

  @override
  FireblendQuery endAt(value, {String key}) {
    return JtQuery(this._query.endAt(value, key));
  }

  @override
  FireblendQuery equalTo(value, {String key}) {
    return JtQuery(this._query.equalTo(value, key));
  }

  @override
  FireblendQuery limitToFirst(int limit) {
    return JtQuery(this._query.limitToFirst(limit));
  }

  @override
  FireblendQuery limitToLast(int limit) {
    return JtQuery(this._query.limitToLast(limit));
  }

  @override
  FireblendQuery orderByChild(String key) {
    return JtQuery(this._query.orderByChild(key));
  }

  @override
  FireblendQuery orderByKey() {
    return JtQuery(this._query.orderByKey());
  }

  @override
  FireblendQuery orderByPriority() {
    return JtQuery(this._query.orderByPriority());
  }

  @override
  FireblendQuery orderByValue() {
    return JtQuery(this._query.orderByValue());
  }

  @override
  FireblendDatabaseReference reference() {
    return JtDbReference(this._query.ref);
  }

  @override
  String getPath() {
    throw Exception('Not implemented!');
  }

  @override
  Map<String, dynamic> getParameters() {
    throw Exception('Not implemented!');
  }
}
