import 'package:firebase_dart/firebase_core.dart';
import 'package:jt_firebase/src/api/data_snapshot.dart';
import 'package:jt_firebase/src/api/database.dart';
import 'package:test/test.dart';

void main() {
  group('[JtDatabase]', () {
    JtDatabase db;
    final dbURL = 'http://quizell-f89b3.firebaseio:9000';
    final path = 't-subjects';
    final child = '-my-custom-id';
    final data = {'01': 'test data', '02': 'data-02', '03': 'data-03'};
    final List<String> listData = ['String 1', 'String 2', 'String 3'];

    test('.fromURL($dbURL)', () async {
      db = JtDatabase.fromURL(dbURL);

      expect(db.runtimeType, JtDatabase);
      expect(db.app.runtimeType, FirebaseApp);
      expect(db.databaseURL, dbURL);
    });

    test(".path('$path').remove()", () async {
      assert(db != null, 'App must be initialized!');

      Exception error;
      await db.ref(path).remove().catchError((err) => error = err);

      expect(error, null);
    });

    test(".path('$path').push('$data')", () async {
      assert(db != null, 'App must be initialized!');

      Exception error;
      await db.ref(path).push(data).catchError((err) => error = err);

      expect(error, null);
    });

    test(".path('$path').once()", () async {
      assert(db != null, 'App must be initialized!');

      final snap = await db.ref(path).once();

      expect(snap.runtimeType, JtDataSnapshot);
      expect(snap.key.length, 10);
      expect((snap.value as Map).values.length, 1);
      expect((snap.value as Map).values.first.length, data.length);
      expect((snap.value as Map).values.first, data);
    });

    test(".path('$path').set($data)", () async {
      assert(db != null, 'App must be initialized!');

      Exception error;
      await db
          .ref(path)
          .child(child)
          .set(data)
          .catchError((err) => error = err);

      expect(error, null);
    });

    test("ref('$path').child('$child').once()", () async {
      assert(db != null, 'App must be initialized!');

      final snap = await db.ref(path).child(child).once();

      expect(snap.runtimeType, JtDataSnapshot);
      expect(snap.key, child);
      expect((snap.value as Map).values.length, 3);
    });

    test("ref('$path').child('$child').set($listData)", () async {
      assert(db != null, 'App must be initialized!');

      await db.ref(path).child(child).set(listData);
      final snap = await db.ref(path).child(child).once();

      expect(snap.runtimeType, JtDataSnapshot);
      expect(snap.key, child);
      expect((snap.value as Map).values, listData);
    });
  });
}
